<?php

    /*

        Yellow Duck Framework version 2.0
        (c) Copyright 2002-2005 Pieter Claerhout

        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2.1 of the License, or (at your option) any later version.

        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    */

    // Check if the framework is loaded
    if ( ! defined( 'YD_FW_NAME' ) ) {
        die( 'Yellow Duck Framework is not loaded.' );
    }

    // Configure the default for this class
    YDConfig::set( 'YD_HTTP_USES_GZIP', 1, false );
    YDConfig::set( 'YD_HTTP_CACHE_TIMEOUT', 3600, false );
    YDConfig::set( 'YD_HTTP_CACHE_USEHEAD', 1, false );

    // Includes
    include_once( YD_DIR_HOME_CLS . '/YDRequest.php' );

    /**
     *  This class defines a url.
     */
    class YDUrl extends YDBase {

        /**
         *	This is the class constructor for the YDUrl class. You need to specify the URL as it's only argument.
         *
         *	@param $url	The url you want to have as a YDUrl object.
         */
        function YDUrl( $url ) {

            // Initialize YDBase
            $this->YDBase();

            // Save the path
            $this->_url = $url;

            // Set the defaults
            $this->defaults = array(
                'scheme' => 'http', 'host' => 'localhost', 'port' => '', 'user' => '', 'pass' => '', 'path' => '',
                'query' => '', 'fragment' => ''
            );

            // Parse the URL
            $this->_url_parsed = array_merge( $this->defaults, parse_url( $url ) );

            // Fix W3C type of URLs
            $this->_url_parsed['query'] = str_replace( '&amp;', '&', $this->_url_parsed['query'] );

            // Parse the query string
            parse_str( $this->_url_parsed['query'], $this->_url_parsed['query'] );

        }

        /**
         *	This function will check if the argument is one of the directories specified in the URL path. It will
         *	basically check if the argument is in the array resulting from the getPathDirectories function.
         *
         *	@param	$arg	Directory to check.
         *
         *	@returns	Boolean indicating if the argument is one of the directories in the URL path or not.
         */
        function isDirectory( $arg ) {
            return in_array( $arg, $this->getPathDirectories() );
        }

        /**
         *	This function will return the original URL.
         *
         *	@returns	The original URL.
         */
        function getUrl() {

            // Add the scheme
            $url = $this->getNamedPart( 'scheme' ) . '://';

            // Add the user information
            if ( $this->getNamedPart( 'user' ) != '' ) {
                $url .= $this->getNamedPart( 'user' );
                if ( $this->getNamedPart( 'pass' ) != '' ) {
                    $url .= ':' . $this->getNamedPart( 'pass' );
                }
                $url .= '@';
            }

            // Add the host information
            $url .= $this->getNamedPart( 'host' );

            // Add the port information
            if ( $this->getNamedPart( 'port' ) != '' ) {
                $url .= ':' . $this->getNamedPart( 'port' );
            }

            // Add the URI
            $url .= $this->getUri();

            // Return the URL
            return $url;

        }

        /**
         *	This function will return a named part of the URL. The following names are recognized: scheme, host, port,
         *	user, pass, path, query and fragment. This function will return an empty string if the part doesn't exist
         *	or if an invalid part has been specified.
         *
         *	@param $name	The name of the part.
         *
         *	@returns	The textual contents of the indicated part.
         *
         *	@internal
         */
        function getNamedPart( $name ) {
            if ( isset( $this->_url_parsed[$name] ) ) {
                return $this->_url_parsed[$name];
            } else {
                return '';
            }
        }

        /**
         *	This function will set the named part to the new contents.
         *
         *	@param $name	The name of the part to update
         *	@param $value	The new value for the part
         */
        function setNamedPart( $name, $value ) {
            $name = strtolower( $name );
            if ( in_array( $name, array_keys( $this->defaults ) ) ) {
                $this->_url_parsed[$name] = $value;
            }
        }

        /**
         *	This function will return the scheme of the URL.
         *
         *	@returns	The scheme of the URL (http, ftp, ...).
         */
        function getScheme() {
            return strtolower( $this->getNamedPart( 'scheme' ) );
        }

        /**
         *	This function will return the host part of the URL.
         *
         *	@returns	The host part of the URL.
         */
        function getHost() {
            return strtolower( $this->getNamedPart( 'host' ) );
        }

        /**
         *	This function will return the port part of the URL. It will return 80 by default.
         *
         *	@returns	The port part of the URL.
         */
        function getPort() {
            $port = $this->getNamedPart( 'port' );
            if ( is_numeric( $port ) ) {
                return intval( $port );
            } else {
                return 80;
            }
        }

        /**
         *	This function will return the user part of the URL.
         *
         *	@returns	The user part of the URL.
         */
        function getUser() {
            return $this->getNamedPart( 'user' );
        }

        /**
         *	This function will return the password part of the URL.
         *
         *	@returns	The password part of the URL.
         */
        function getPassword() {
            return $this->getNamedPart( 'pass' );
        }

        /**
         *	This function will return the path part of the URL.
         *
         *	@returns	The path part of the URL.
         */
        function getPath() {
            return $this->getNamedPart( 'path' );
        }

        /**
         *	This function will return the path part of the URL as an array.
         *
         *	@returns	The path part of the URL.
         */
        function getPathSegments() {
            $segments = explode( '/', $this->getNamedPart( 'path' ) );
            return array_slice( $segments, 1 );
        }

        /**
         *	This function will return the list of path segments that appear after the first occurence of the indicated
         *	directory/file in the path part of the string.
         *
         *	@code
         *	$url = new YDUrl( 'http://www.yellowduck.be/ydf2/forum/cool.html' );
         *	$r = $url->getSubDirectories('ydf2');   // returns: ['forum', 'cool.html'];
         *	$r = $url->getSubDirectories('forum');  // returns: ['cool.html'];
         *	$r = $url->getSubDirectories('test');   // returns: [];
         *
         *	// it gets the sub-directories of the first ocurrence:
         *	$url = new YDUrl( 'http://www.yellowduck.be/ydf2/forum/ydf2/forum/cool.html' );
         *	$r = $url->getSubDirectories('ydf2');   // returns: ['forum', 'ydf2', 'forum', 'cool.html'];
         *	$r = $url->getSubDirectories('forum');  // returns: ['ydf2', 'forum', 'cool.html'];
         *	$r = $url->getSubDirectories('test');   // returns: [];
         *	@endcode
         *
         *	@param	$item	The items to get the subsegments from.
         *
         *	@returns	Array with the subsegments that appear after the first occurence of the indicated items in
         *	the path part of the string
         */
        function getPathSubsegments( $item ) {

            // Get the list of path segments in the path string
            $segments = $this->getPathSegments();

            // Get the position of the item in the list of subsegments
            $pos = array_search( $item, $segments );

            // Return an empty array if not found
            if ( $pos === false ) {
                return array();
            }

            // If found, return the list of subsegments
            return array_slice( $segments, $pos+1 );

        }

        /**
         *	This function will return the path part of the URL as an array, only including directories. If the last part
         *	of the URL is not a forward slash, the last part is considered to be a regular file.
         *
         *	@returns	The path part of the URL.
         */
        function getPathDirectories() {
            $dirs = $this->getPathSegments();
            return array_slice( $dirs, 0, -1 );
        }

        /**
         *	This function will return the list of directories that appear after the first occurence of the indicated
         *	directory in the path part of the string. The following code examples show you how this can be used:
         *
         *	@code
         *	$url = new YDUrl( 'http://www.yellowduck.be/ydf2/forum/cool.html' );
         *	$r = $url->getPathSubdirectories('ydf2');   // returns: ['forum'];
         *	$r = $url->getPathSubdirectories('forum');  // returns: [];
         *	$r = $url->getPathSubdirectories('test');   // returns: [];
         *
         *	// it gets the sub-directories of the first ocurrence:
         *	$url = new YDUrl( 'http://www.yellowduck.be/ydf2/forum/ydf2/forum/cool.html' );
         *	$r = $url->getPathSubdirectories('ydf2');   // returns: ['forum', 'ydf2', 'forum'];
         *	$r = $url->getPathSubdirectories('forum');  // returns: ['ydf2', 'forum'];
         *	$r = $url->getPathSubdirectories('test');   // returns: [];
         *	@endcode
         *
         *	@param	$dir	The directory to get the subdirectories from.
         *
         *	@returns	Array with the directories that appear after the first occurence of the indicated directory in
         *	the path part of the string
         */
        function getPathSubdirectories( $dir ) {

            // Get the list of directories in the path string
            $dirs = $this->getPathDirectories();

            // Get the position of the directory in the list of directories
            $pos = array_search( $dir, $dirs );

            // Return an empty array if not found
            if ( $pos === false ) {
                return array();
            }

            // If found, return the list of subdirectories
            return array_slice( $dirs, $pos+1 );

        }

        /**
         *	This function will return the query part of the URL. This is everything after the ? mark.
         *
         *	@returns	The query part of the URL.
         */
        function getQuery() {
            return $this->getNamedPart( 'query' );
        }

        /**
         *	This function will return the fragment part of the URL. This is everything after the # mark.
         *
         *	@returns	The fragment part of the URL.
         */
        function getFragment() {
            return $this->getNamedPart( 'fragment' );
        }

        /**
         *	This function will return the URI part of the URL.
         *
         *	@returns The URI parts of the URL.
         */
        function getUri() {

            // Build the query string
            $querystr = '';
            foreach ( $this->getNamedPart( 'query' ) as $key=>$value ) {
                if ( is_array( $value ) ) {
                    foreach ( $value as $key1=>$val ) {
                        $querystr .= ( strlen( $querystr ) < 1 ) ? '' : '&amp;';
                        $querystr .= rawurlencode( $key  . '[' . $key1 . ']' ) . '=' . rawurlencode( $val );
                    }
                } else {
                    $querystr .= ( strlen( $querystr ) < 1 ) ? '' : '&amp;';
                    $querystr .= rawurlencode( $key ) . '=' . rawurlencode( $value );
                }
            }

            // Build the URI
            $uri = $this->getNamedPart( 'path' ) ? $this->getNamedPart( 'path' ) : '';
            $uri .= $querystr ? '?'.$querystr : '';
            $uri .= $this->getNamedPart( 'fragment' ) ? '#'.$this->getNamedPart( 'fragment' ) : '';

            // Return the URI
            return $uri;

        }

        /**
         *	This function will retrieve the contents of a named query string variable. It will return an empty string if
         *	the named query variable is not existing. If the name of the query variable indicates that it should be an
         *	array, it will makes sure an array is returned.
         *
         *	@param	$name	The name of the query variable to retrieve.
         *
         *	@returns	The contents of the query variable.
         */
        function getQueryVar( $name ) {
            if ( isset( $this->_url_parsed['query'][$name] ) ) {
                if ( substr( $name, -2, 2 ) == '[]' ) {
                    if ( ! is_array( $this->_url_parsed['query'][$name] ) ) {
                        $this->_url_parsed['query'][$name] = array( $this->_url_parsed['query'][$name] );
                    }
                }
                return $this->_url_parsed['query'][$name];
            } else {
                return ( substr( $name, -2, 2 ) == '[]' ) ? array() : '';
            }
        }

        /**
         *	This function will set the indicated query variable to the new value. If the name of the variable indicates
         *	that the value is an array, this function will convert it to an array if it's not an array.
         *
         *	If the query variable is already existing, it's value will be updated. If it's not existing yet, the query
         *	variable will be created.
         *
         *	@param	$name	The name of the query variable to update or set.
         *	@param	$val	The value for the query variable.
         */
        function setQueryVar( $name, $val ) {
            if ( substr( $name, -2, 2 ) == '[]' ) {
                $val = is_array( $val ) ? $val : array( $val );
            }
            $this->_url_parsed['query'][$name] = $val;
        }

        /**
         *	This function will delete the indicated query variable if it exists.
         *
         *	@param	$name	The name of the query variable to delete.
         */
        function deleteQueryVar( $name ) {
            unset( $this->_url_parsed['query'][$name] );
        }

        /**
         *	Function to get the contents of the URL. It will get the contents using Gzip compression if possible in
         *	order to save bandwidth. It uses the HTTP Client class from Simon Willison to do the dirty work.
         *
         *	More information about the HTTP client class can be found on: http://scripts.incutio.com/httpclient/
         *
         *	If it fails to retrieve the data, it will raise a fatal error.
         *
         *	By default, it will cache the downloaded data based on the etag and last-modified headers. The cache files
         *	are stored in the temp directory of the Yellow Duck framework and have the extension "wch". You can delete
         *	these automatically as they will be recreated on the fly if needed.
         *
         *	For configuring the cache, there are two configuration variables you can redefine if needed:
         *	YD_HTTP_CACHE_TIMEOUT: the lifetime of the cache in seconds (default: 3600).
         *	YD_HTTP_CACHE_USEHEAD: if a HEAD HTTP request should be used to verify the cache validity (default: 1).
         *
         *	@param $cache	(optional) Indicate if the web content should be cached or not. By default, caching is
         *					turned on.
         *  @param $fail    (optional) Whether to fail or not if the contents cannot be downloaded. Defaults to true.
         *
         *	@returns	Returns the contents of the URL.
         */
        function getContents( $cache=true, $fail=true ) {

            // Check if caching is enabled
            $cacheFName = null;

            // Check the cache
            if ( $cache == true ) {

                // Include the filesystem library
                include_once( YD_DIR_HOME_CLS . '/YDFileSystem.php');

                // Check if we need to use the HTTP HEAD function
                if ( YDConfig::get( 'YD_HTTP_CACHE_USEHEAD' ) == 1 ) {

                    // Get the headers
                    $headers = $this->getHeaders();

                    // Check if we have etag or last modified
                    if ( isset( $headers['etag'] ) || isset( $headers['last-modified'] ) ) {
                        $cacheFName = $this->getUrl();
                        if ( isset( $headers['etag'] ) ) {
                            $cacheFName .= $headers['etag'];
                        }
                        if ( isset( $headers['last-modified'] ) ) {
                            $cacheFName .= $headers['last-modified'];
                        }
                        if ( isset( $headers['content-length'] ) ) {
                            $cacheFName .= $headers['content-length'];
                        }
                        $cacheFName = YD_TMP_PRE . 'W_' . md5( $cacheFName ) . '.wch';
                        $cacheFName = YD_DIR_TEMP . '/' . $cacheFName;
                    }

                }

                // If the cache filename is null, use the default one
                if ( $cacheFName == null ) {
                    $cacheFName = YD_DIR_TEMP . '/' . YD_TMP_PRE . 'W_' . md5( $this->getUrl() ) . '.wch';
                }

                // Use the cache file if any
                if ( is_file( $cacheFName ) ) {
                    $file = new YDFSFile( $cacheFName );
                    $cacheValidTime = $file->getLastModified() + YDConfig::get( 'YD_HTTP_CACHE_TIMEOUT' );
                    if ( time() < $cacheValidTime ) {
                        return $file->getContents();
                    }
                };

            }

            // Create a new HTTP client
            $client = $this->_getHttpClient();

            // Now send the request
            $result = @ $client->doRequest();

            // Check if there was a result
            if ( $result == false ) {
                if ( $fail ) {
                    trigger_error(
                        'Failed to retrieve the data from the url "' . $this->getUrl() . '". ' . $client->getError(),
                        YD_ERROR
                    );
                } else {
                    return false;
                }
            } else {
                $data = @ $client->getContent();
            }

            // Check if caching is enabled
            if ( $cache == true ) {

                // Save the cached data
                if ( $cacheFName != null ) {
                    $dir = new YDFSDirectory( YD_DIR_TEMP );
                    $dir->createFile( $cacheFName, $data );
                }

            }

            // Return the data
            return $data;

        }

        /**
         *	Function to get the contents of the URL and automatically applies a regular expression to the result. This
         *	function is based on the preg library and requires the preg syntax to function properly. If it fails to
         *	retrieve the data, it will raise a fatal error. If no port was specified in the URL, it will default to 80.
         *
         *	@param $regex	The regular expression you want to apply to the the contents of the URL.
         *	@param $cache	(optional) Indicate if the web content should be cached or not. By default, this is turned on.
         *
         *	@returns	Array with the regex matches from the URL contents.
         */
        function getContentsWithRegex( $regex, $cache=true ) {
            $contents = $this->getContents( $cache );
            preg_match_all( $regex, $contents, $matches );
            return $matches;
        }

        /**
         *	This function retrieves the header information for the specified URL.
         *
         *	@return	Array containing the headers for the URL.
         */
        function getHeaders() {
            $client = $this->_getHttpClient();
            $client->method = 'HEAD';
            $client->headers_only = true;
            $result = @ $client->doRequest();
            if ( $result == false ) {
                trigger_error(
                    'Failed to retrieve the data from the url "' . $this->getUrl() . '". ' . $client->getError(),
                    YD_ERROR
                );
            } else {
                return $client->headers;
            }
        }

        /**
         *	This function retrieves the header information for the specified URL.
         *
         *	@return	Array containing the headers fors the URL.
         */
        function getStatus() {
            $client = $this->_getHttpClient();
            $client->method = 'HEAD';
            $client->headers_only = true;
            $result = @ $client->doRequest();
            if ( $result == false ) {
                trigger_error(
                    'Failed to retrieve the data from the url "' . $this->getUrl() . '". ' . $client->getError(),
                    YD_ERROR
                );
            } else {
                return intval( $client->status );
            }
        }

        /**
         *  This function will convert all relative URLs to absolute ones using the given base.
         *
         *  @param  $text   The text in which you want to have the links converted to absolute ones.
         *  @param  $base   (optional) The base url. By default, it uses the current url.
         *
         *  @returns    The text with all links converted to absolute ones.
         *
         *  @static
         */
        function makeLinksAbsolute( $text, $base=null ) {
            while ( eregi( "(href|src|action)=\"(([^/])[[:alnum:]/+=%&_.~?-]*)\"", $text, $regs ) ) {
                $input_uri = str_replace( '?', '\?', $regs[2] );
                $output_uri = YDUrl::makeLinkAbsolute( $regs[2], $base );
                $text = eregi_replace( "((href|src|action)=\")$input_uri(\")", "\\1$output_uri\\3", $text );
            }
            return $text;
        }

        /**
         *  Helper function for the makeLinksAbsolute function.
         *
         *  @param  $rel_uri    The uri to convert to an absolute one.
         *  @param  $base       (optional) The base url. By default, it uses the current url.
         *
         *  @returns    The link converted to an absolute one.
         *
         *  @static
         */
         function makeLinkAbsolute( $rel_uri, $base=null ) {
            if ( substr( strtolower( $rel_uri ), 0, 7 ) == 'http://' ) {
                return rtrim( $rel_uri, '/.' );
            }
            if ( is_null( $base ) ) {
                $base = YDRequest::getCurrentUrl( true );
            }
            preg_match( "'^([^:]+://[^/]+)/'", $base, $m );
            $base_start = $m[1];
            if ( preg_match( "'^/'", $rel_uri ) ) {
                return rtrim( $base_start . $rel_uri, '/.' );
            }
            $base = preg_replace( "{[^/]+$}", '', $base );
            $base .= $rel_uri;
            $base = preg_replace( "{^[^:]+://[^/]+}", '', $base );
            $base_array = explode( '/', $base );
            if ( count( $base_array ) && ! strlen( $base_array[0] ) ) {
                array_shift( $base_array );
            }
            $i = 1;
            while ( $i < count( $base_array ) ) {
                if ( $base_array[$i-1] == "." ) {
                    array_splice( $base_array, $i - 1, 1 );
                    if ( $i > 1 ) {
                     $i--;
                    }
                } elseif ( $base_array[$i] == ".." && $base_array[$i - 1]!= ".." ) {
                    array_splice( $base_array, $i - 1, 2 );
                    if ( $i > 1 ) {
                        $i--;
                        if ( $i == count( $base_array ) ) {
                            array_push( $base_array, "" );
                        }
                    }
                } else {
                    $i++;
                }
            }
            if ( count( $base_array ) && isset( $base_array[-1] ) && $base_array[-1] == "." ) {
                $base_array[-1] = "";
            }
            while ( count( $base_array ) && preg_match( "/^\.\.?$/", $base_array[0] ) ) {
                array_shift( $base_array );
            }
            return rtrim( $base_start . '/' . implode( "/", $base_array ), '/.' );
        }

        /**
         *  This function redirects to the URL
         */
        function redirect() {
            header( 'Location: ' . $this->getUrl() );
        }

        /**
         *	This function merges the query string part of another YDUrl object(s) to the current one.
         *
         *	@param	$obj	An YDUrl object or an array of objects.
         */
        function merge( $obj ) {
            
            if ( ! is_array( $obj ) ) {
                $obj = array( $obj );
            }
            
            foreach ( $obj as $url ) {
           
                foreach ( $url->getNamedPart( 'query' ) as $key => $value ) {
                   
                    if ( in_array( $key, array_keys( $this->getNamedPart( 'query' ) ) ) ) {
                       
                        if ( is_array( $this->getQueryVar( $key ) ) && is_array( $value ) ) {
                            $this->setQueryVar( $key, array_merge( $this->getQueryVar( $key ), $value ) );
                        }
                       
                    } else {
                        $this->setQueryVar( $key, $value );
                    }
                }
            }
           
        }

        /**
         *	This function will return an already setup HTTP client object.
         *
         *	@internal
         *
         *	@returns	A new HttpClient class instance.
         */
        function _getHttpClient() {
            include_once( YD_DIR_HOME_CLS . '/YDHttpClient.php');
            if ( $this->getScheme() != 'http' ) {
                trigger_error( 'getContents: Only HTTP URLs are supported.', YD_ERROR );
            }
            $client = new YDHttpClient( $this->getHost(), $this->getPort() );
            $client->path = $this->getUri();
            $client->referer = $this->getUrl();
            $client->method = 'GET';
            return $client;
        }

    }

?>
